/**
 * @license
 * Copyright © 2015 Intuilab
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
 *
 * The Software is provided "as is", without warranty of any kind, express or implied, including but not limited to the warranties of merchantability,
 * fitness for a particular purpose and noninfringement. In no event shall the authors or copyright holders be liable for any claim, damages or other liability,
 * whether in an action of contract, tort or otherwise, arising from, out of or in connection with the Software or the use or other dealings in the Software.
 *
 * Except as contained in this notice, the name of Intuilab shall not be used in advertising or otherwise to promote the sale,
 * use or other dealings in this Software without prior written authorization from Intuilab.
 */

/**
* Inheritance on EventEmitter base class
* @type {EventEmitter}
*/
RandomFromList.prototype = new EventEmitter();        // Here's where the inheritance occurs
RandomFromList.prototype.constructor = RandomFromList;

/**
* @constructor
*/
function RandomFromList() {

    this.possibilities = "";
    this.listPossibilities = [];
    this.separator = 'comma';
    this.internalSeparator = ",";

    this.output = "";
}

/**
 * Setters for different properties
 */
RandomFromList.prototype.setPossibilities = function (values) {
    // XXX: !== in order to also compare types an manage null values properly
    if (this.possibilities != values) {
        this.possibilities = values;
        this.emit('possibilitiesChanged', [this.possibilities]);
        this.splitPossibilities();

        //generate a new random value
        this.generate();
    }
};

RandomFromList.prototype.setSeparator = function (separator) {
    // XXX: !== in order to also compare types an manage null values properly
    if (this.separator != separator) {
        //process special values
        switch (separator) {
            case "comma":
                this.internalSeparator = ",";
                break;
            case "blank":
                this.internalSeparator = " ";
                break;
            case "semicolon":
                this.internalSeparator = ";";
                break;
            case "pipe":
                this.internalSeparator = "|";
                break;
            case "dash":
                this.internalSeparator = "-";
                break;
            case "underscore":
                this.internalSeparator = "_";
                break;


        }

        this.separator = separator;
        this.emit('separatorChanged', [this.separator]);
        this.splitPossibilities();

        //generate a new random value
        this.generate();
    }
};

RandomFromList.prototype.splitPossibilities = function () {
    this.listPossibilities = this.possibilities.split(this.internalSeparator);
    this.emit('listPossibilitiesChanged', [this.listPossibilities]);
};

/**
 * Rand value to the result
 *
 */
RandomFromList.prototype.generate = function () {
    var rand = this.listPossibilities.length * Math.random();
    var randIndex = Math.floor(rand);
    if (randIndex >= this.listPossibilities.length){
        randIndex = this.listPossibilities.length - 1;
    }
    this.output = this.listPossibilities[randIndex];
    this.emit('outputChanged', [this.output]);
};

function contains(a, obj) {
    for (var i = 0; i < a.length; i++) {
        // XXX: ensure i is a property (and not a function for instance) as javascript can define function with integers (potentially)
        if (a.hasOwnProperty(i)) {
            if (a[i] === obj) {
                return true;
            }
        }
    }
    return false;
}

RandomFromList.prototype.addItemInPossibilities = function (item) {

    if (!contains(this.listPossibilities, item)) {
        // XXX: !== in order to also compare types an manage null values properly
        if (this.possibilities != "")
            this.possibilities += this.internalSeparator;
        this.possibilities += item;
        this.splitPossibilities();
    }

};

RandomFromList.prototype.clearPossibilities = function () {
    this.possibilities = "";
    this.splitPossibilities();
};
